@@ -7,7 +7,7 @@
 -- IMAPFilter can be detached from the controlling terminal and run in
 -- the background as a system daemon.
 --
--- The auxiliary function become_daemon() is supplied for conveniency.
+-- The auxiliary function become_daemon() is supplied for convenience.
 -- The following example puts imapfilter in the background and runs
 -- endlessly, executing the commands in the forever() function and
 -- sleeping for 600 seconds between intervals:
@@ -28,7 +28,7 @@ become_daemon(600, forever)
 
 while true do
     myaccount.mymailbox:enter_idle()
-    results = myaccount.mymailbox:is_unread()
+    results = myaccount.mymailbox:is_unseen()
     results:move_messages(myaccount.myothermailbox)
 end
 
@@ -47,18 +47,18 @@ end
 
 while true do
     custom_idle(myaccount.mymailbox)
-    results = myaccount.mymailbox:is_unread()
+    results = myaccount.mymailbox:is_unseen()
     results:move_messages(myaccount.myothermailbox)
 end
 
 
 -- IMAPFilter can take advantage of all those filtering utilities that
 -- are available and use a wide range of heuristic tests, text analysis,
--- internet-based realtime blacklists, advanced learning algorithms,
+-- internet-based real-time blacklists, advanced learning algorithms,
 -- etc. to classify mail.  IMAPFilter can pipe a message to a program
 -- and act on the message based on the program's exit status.
 --
--- The auxiliary function pipe_to() is supplied for conveniency.  For
+-- The auxiliary function pipe_to() is supplied for convenience.  For
 -- example if there was a utility named "bayesian-spam-filter", which
 -- returned 1 when it considered the message "spam" and 0 otherwise:
 
@@ -127,3 +127,64 @@ account1 = IMAP {
     username = 'user1',
     password = password
 }
+
+-- Normally when there's a problem, such as network failure to receive data
+-- from or send data to the IMAP server, an error is raised, IMAPFilter stops
+-- execution of the user's config, and quits.
+--
+-- In previous versions, IMAPFilter had an option to enable automatic recovery,
+-- in which case it tried to reconnect, repeat the last IMAP command, and then
+-- continue execution of the user's config.
+--
+-- Recently, this functionality was replaced by a new, more advanced, robust,
+-- and flexible mechanism, where recovery can be fully controlled by the user,
+-- using the auxiliary recover() function.
+--
+-- This example shows that set of commands can be grouped together in a
+-- function, and if there's an error, the function will be re-executed from
+-- start, once a connection has been re-established with the server:
+
+function commands()
+    results = myaccount.mymailbox:is_old()
+    results:move_messages(myaccount.myothermailbox)
+end
+
+recover(commands)
+
+-- When there are multiple accounts, where a series of actions are taking place
+-- repeatedly, recovery can be controlled in such a way, that the rest of the
+-- accounts commands are still processed, and the failure of a single account,
+-- doesn't block the rest:
+
+function commands1()
+    results = myaccount.mymailbox:is_old()
+    results:move_messages(myaccount.myothermailbox)
+end
+
+function commands2()
+    results2 = myaccount2.mymailbox2:is_seen()
+    results2:delete_messages()
+end
+
+while true do
+    recover(commands1, 4)
+    recover(commands2, 2)
+end
+
+-- It is also possible to handle the error captured during failure, or even
+-- further process the returned values of the function executed:
+
+function commands1()
+    results = myaccount.mymailbox:is_old()
+    results:move_messages(myaccount.myothermailbox)
+end
+
+function commands2()
+    return myaccount2.mymailbox2:is_seen()
+end
+
+success, errormsg = recover(commands1, 5)
+if not success then print(errormsg) end
+
+success2, results2 = recover(commands2)
+if success2 then results2:delete_messages() end
