@@ -7,12 +7,6 @@
 
 /* IMAP session. */
 typedef struct session {
-	const char *server;	/* Server hostname. */
-	const char *port;	/* Server port. */ 
-	const char *sslproto;	/* SSL protocol. */
-	const char *username;	/* User name. */
-	const char *password;	/* User password. */
-        const char *oauth2;     /* Oauth2 token */
 	int socket;		/* Socket. */
 	SSL *sslconn;		/* SSL connection. */
 	unsigned int protocol;	/* IMAP protocol.  Currently IMAP4rev1 and
@@ -22,7 +16,6 @@ typedef struct session {
 		char *prefix;	/* Namespace prefix. */
 		char delim;	/* Namespace delimiter. */
 	} ns;
-	char *selected;	/* Selected mailbox. */
 } session;
 
 
